1/*********************************************************************
*						   ECO_Module
**********************************************************************
* File Name: 	LCD.c
* MCU: 			18F4321
* Compiler:		MPLAB C18 v3.0
* Company:		Rostech Innovative Industries
* Author:		CJ Della Porta
* Org. Date:	05/24/09
* Version:		0.1
* Level:		1st Tier 
**********************************************************************
* Functional Description:
* This file contains all of the LCD functions that will be called from
* the main function.
**********************************************************************
* Revision History:
* Ver		Date		Description
*=====================================================================
* --		05/24/09		Prototype - LCD Functions
*=====================================================================

/** INCLUDES *********************************************************/
#include <IO_cfg.h>
#include <LCD.h>
#include <delays.h>                                      
#include "p18f4321.h"

/** CONFIGURATION BITS ***********************************************/
/* CONFIG2H (300003h) */
//#pragma config WDT = OFF, WDTPS = 4

/** MACROS ***********************************************************/                                        
/* See IO_cfg.h */

/** Constants ********************************************************/
/* See IO_cfg.h */

/** VARIABLES ********************************************************/
static unsigned char break_point;

/** PROTOTYPES *******************************************************/
/* Found in LCD.h */

/** LCD Init *********************************************************/
void LCD_Init(void)
{

	/*Verify that enable, RS, and RW are cleared */
	LCD_EN = CLEAR;
	LCD_RS = CLEAR;
	LCD_RW = CLEAR;
	
	/* Set data pins to write */
	LCD_WRITE();

	/* Wait 15 ms (at least 15 ms) */
	Delay100TCYx(38);	//Delay for approx. 15ms (38*(100*Tcy(4us)) = 15.2ms) 

	/* Send initialization data half-byte */
	LCD_Bits(0b00000011);
	LCD_EN = SET;
	Delay10TCYx(1);	//Delay for 40us
	LCD_EN = CLEAR;

	/* Wait 8 ms (at least 4.1 ms) */
	Delay100TCYx(20);	//Delay (20*(100*Tcy(4us)) = 8ms)

	/* Send initialization data half-byte */
	LCD_Bits(0b00000011);		
	LCD_EN = SET;
	Delay10TCYx(1);	//Delay for 40us
	LCD_EN = CLEAR;

	/* Wait 0.5 ms (at least 100 us) */
	Delay10TCYx(13);	//Delay (13*(10*Tcy(4us)) = 0.52ms)

	/* Send initialization data half-byte */
	LCD_Bits(0b00000011);			
	LCD_EN = SET;
	Delay10TCYx(1);	//Delay for 40us
	LCD_EN = CLEAR;

	/* Wait 80us */
	Delay10TCYx(2);	//Delay (2*(10*Tcy(4us)) = 0.08ms)

	/* Set interface to 4-bit mode */
	LCD_Bits(LCD_4BITMODE);		
	LCD_EN = SET;
	Delay10TCYx(1);		//Delay for 40us	
	LCD_EN = CLEAR;
	
	/* Wait 80us */
	Delay10TCYx(2);	//Delay (2*(10*Tcy(4us)) = 0.08ms)

	/* Function set - Set line # and Font Type */
	LCD_Bits(0b00000010);			
	LCD_EN = SET;
	Delay10TCYx(1);		//Delay for 40us	
	LCD_EN = CLEAR;
	Delay10TCYx(2);		//Delay for 40us
	LCD_Bits(0b00001000);		
	LCD_EN = SET;
	Delay10TCYx(1);		//Delay for 40us
	LCD_EN = CLEAR;

	/* Wait 80us */
	Delay10TCYx(2);	//Delay (2*(10*Tcy(4us)) = 0.08ms)

	/* Function set - Display Off */
	LCD_Bits(0b00000000);		
	LCD_EN = SET;
	Delay10TCYx(1);		//Delay for 40us
	LCD_EN = CLEAR;
	Delay10TCYx(1);		//Delay for 40us
	LCD_Bits(0b00001000);
	LCD_EN = SET;
	Delay10TCYx(1);		//Delay for 40us
	LCD_EN = CLEAR;

	/* Wait 80us */
	Delay10TCYx(2);	//Delay (2*(10*Tcy(4us)) = 0.08ms)

	/* Function set - Display Clear */
	LCD_Bits(0b00000000);		
	LCD_EN = SET;
	Delay10TCYx(1);		//Delay for 40us
	LCD_EN = CLEAR;
	Delay10TCYx(1);		//Delay for 40us
	LCD_Bits(0b00000001);
	LCD_EN = SET;
	Delay10TCYx(1);		//Delay for 40us
	LCD_EN = CLEAR;

	/* Wait 80us */
	Delay10TCYx(2);	//Delay (2*(10*Tcy(4us)) = 0.08ms)

	/* Function set - Entry Mode Select */
	LCD_Bits(0b00000000);		
	LCD_EN = SET;
	Delay10TCYx(1);		//Delay for 40us
	LCD_EN = CLEAR;
	Delay10TCYx(1);		//Delay for 40us
	LCD_Bits(0b00000010);
	LCD_EN = SET;
	Delay10TCYx(1);		//Delay for 40us
	LCD_EN = CLEAR;

	/* Wait 80us */
	Delay10TCYx(100);	//Delay (2*(10*Tcy(4us)) = 0.08ms)

}//end of LCD Init

/** LCD Bits *********************************************************/
void LCD_Bits(unsigned char data)
{
	/* Pass the 4 LSB into the data registers */
	LATD &= 0b11110000;
	LATD |= data;

}//end of LCD Bits

/** LCD Instruction **************************************************/
/* This function will send the upper 4 bits first                    */
void LCD_Instruction(unsigned char command)
{

	unsigned char upper, lower;

	/* Mask out upper 4 bits */
	upper = (command)&(0b11110000);

	/* Shift upper bits into position */
	upper >> 4;

	/* Wait for LCD to be ready */
	while(LCD_Busy()){}
	
	/* Load Data into Pins */
	LCD_WRITE();		//set pins to write output
	LCD_Bits(upper);
	
	/* Set to instruction write mode */
	LCD_RS = CLEAR;
	LCD_RW = CLEAR;	

	/* Transmit Data */
	LCD_EN = SET;
	Delay10TCYx(2);		//Delay for 80us
	LCD_EN = CLEAR;
	Delay10TCYx(2);		//Delay for 80us

	/* Mask out lower 4 bits */
	lower = (command)&(0b00001111);
	
	LCD_Bits(lower);	

	/* Transmit Data */
	LCD_EN = SET;
	Delay10TCYx(2);		//Delay for 40us
	LCD_EN = CLEAR;
	Delay10TCYx(100);		//Delay for 4ms;

}//end of LCD Instruction

/** LCD wData **************************************************/
/* This function will send the upper 4 bits first             */
void LCD_wData(unsigned char data)
{

	unsigned char upper, lower;

	/* Mask out upper 4 bits */
	upper = (data)&(0b11110000);

	/* Shift upper bits into position */
	upper >> 4;

	/* Wait for LCD to be ready */
	while(LCD_Busy()){}
	
	/* Load Data into Pins */
	LCD_WRITE();		//set pins to write output
	LCD_Bits(upper);
	
	/* Set to data write mode */
	LCD_RS = SET;
	LCD_RW = CLEAR;	
	Delay10TCYx(1);		//Delay for 40us
	/* Transmit Data */
	LCD_EN = SET;
	Delay10TCYx(1);		//Delay for 40us
	LCD_EN = CLEAR;
	Delay10TCYx(10);		//Delay for 40us

	/* Mask out lower 4 bits */
	lower = (data)&(0b00001111);
	
	LCD_Bits(lower);	

	/* Transmit Data */
	LCD_EN = SET;
	Delay10TCYx(1);		//Delay for 40us
	LCD_EN = CLEAR;
	Delay10TCYx(10);		//Delay for 40us;

}//end of LCD rData

/** LCD Set DDRAM ***************************************************/
void LCD_Set_DDRAM(unsigned char address)
{
	
	LCD_Instruction(0b10000000|address);

}//end LCD Set DDRAM


/** LCD Busy ********************************************************/
unsigned int LCD_Busy(void)
{

	/* Create dat variable */
	unsigned char data;

	/* Set the data pins to input */
	LCD_READ();
	
	/* Set to BF mode */
	LCD_RS = CLEAR;
	LCD_RW = SET;

	/* Transmist Data */
	LCD_EN = SET;
	Debounce_Delay(1);
	data = LCD_DB7_I;
	LCD_EN = CLEAR;
	
	/* Check the busy flag */
	return (data);		//returns 1 for busy

}//end of LCD Busy

/** EOF LCD.c *******************************************************/
