/*********************************************************************
*						   ECO_Module
**********************************************************************
* File Name: 	LCD.c
* MCU: 			18F4321
* Compiler:		MPLAB C18 v3.0
* Company:		Rostech Innovative Industries
* Author:		CJ Della Porta
* Org. Date:	05/24/09
* Version:		0.1
* Level:		1st Tier 
**********************************************************************
* Functional Description:
* This file contains all of the LCD functions that will be called from
* the main function.
**********************************************************************
* Revision History:
* Ver		Date		Description
*=====================================================================
* --		05/24/09		Prototype - LCD Functions
*=====================================================================*/

#ifndef __LCD_H
#define __LCD_H
                                    
/** MACROS ***********************************************************/                                        
/* See IO_cfg.h */

/** LCD Initialization Settings **************************************/
#define LCD_4BITMODE 0b00000010			//4 bit mode (use only lower 4 bits)
#define LCD_LINE_FONT 0b00101100		//Bit3 - line num, Bit4 - font type
#define LCD_ENTRY_MODE 0b00000100		//bit1 - I/D, bit0 - SH

/** LCD Commands *****************************************************/
#define LCD_CLEAR_DISPLAY 0b00000001	//Clear the display
#define LCD_RETURN_HOME 0b00000010		//Return home
#define LCD_ALL_OFF 0b00001000			//Disable display
#define LCD_BLINK_CURSOR 0b00001111		//Blinking cursor
#define LCD_STEADY_CURSOR 0b00001110	//Steady cursor
#define LCD_CURSOR_RSHIFT 0b00010100	//Shift the cursor right

/** Macro Functions ****************************************************/
#define LCD_READ() TRISD |= 0b00001111
#define LCD_WRITE() TRISD &= 0b11110000

/** LCD Character Constants ******************************************/
#define LCD_SPACE 0b0010000				//white space character
#define LCD_EXC 0b00100001				//exclamation point	!
#define LCD_DQUOTE 0b00100010			//double quotation mark	"
#define LCD_HASH 0b00100011				//hash mark #
#define LCD_DOLLAR 0b00100100			//dollar sign $
#define LCD_PERCENT 0b00100101			//percent sign %
#define LCD_AMPERSAND 0b00100110		//ampersand sign &
#define LCD_SQUOTE 0b00100111			//single quotation mark '
#define LCD_OPENPAR 0b00101000			//open parenthesis (
#define LCD_CLOSEPAR 0b00101001			//close parenthesis )
#define LCD_STAR 0b00101010				//star symbol *
#define LCD_PLUS 0b001001011			//plus symbol +
#define LCD_COMMA 0b00101100			//comma symbol ,
#define LCD_DASH 0b00101101				//dash (minus) symbol -
#define LCD_PERIOD 0b00101110			//period .
#define LCD_FSLASH 0b00101111			//forward slash /
#define LCD_0 0b00110000				//zero 0
#define LCD_1 0b00110001				//one 1
#define LCD_2 0b00110010				//two 2
#define LCD_3 0b00110011				//three 3
#define LCD_4 0b00110100				//four 4
#define LCD_5 0b00110101				//five 5
#define LCD_6 0b00110110				//six 6
#define LCD_7 0b00110111				//seven 7
#define LCD_8 0b00111000				//eight 8
#define LCD_9 0b00111001				//nine 9
#define LCD_COLON 0b00111010			//colon :
#define LCD_SCOLON 0b00111011			//semi-colon ;
#define LCD_LESS 0b00111100				//less-than symbol <
#define LCD_EQ 0b00111101				//equal sign =
#define LCD_GREATER 0b00111110			//great-than symbol >
#define LCD_QUESTION 0b00111111			//question mark ?
#define LCD_AT 0b01000000				//commercial at @
#define LCD_A 0b01000001				//A
#define LCD_B 0b01000010				//B
#define LCD_C 0b01000011				//C
#define LCD_D 0b01000100				//D
#define LCD_E 0b01000101				//E
#define LCD_F 0b01000110				//F
#define LCD_G 0b01000111				//G
#define LCD_H 0b01001000				//H
#define LCD_I 0b01001001				//I
#define LCD_J 0b01001010				//J
#define LCD_K 0b01001011				//K
#define LCD_L 0b01001100				//L
#define LCD_M 0b01001101				//M
#define LCD_N 0b01001110				//N
#define LCD_O 0b01001111				//O
#define LCD_P 0b01010000				//P
#define LCD_Q 0b01010001				//Q
#define LCD_R 0b01010010				//R
#define LCD_S 0b01010011				//S
#define LCD_T 0b01010100				//T
#define LCD_U 0b01010101				//U
#define LCD_V 0b01010110				//V
#define LCD_W 0b01010111				//W
#define LCD_X 0b01011000				//X
#define LCD_Y 0b01011001				//Y
#define LCD_Z 0b01011010				//Z
#define LCD_OPENBRACK 0b01011011		//open bracket [
#define LCD_CLOSEBRACK 0b01011101		//close bracket ]
#define LCD_CARET 0b01011110			//caret symbol ^
#define LCD_UNDERSCORE 0b01011111		//underscore symbole _
#define LCD_a 0b01100001				//a
#define LCD_b 0b01100010				//b
#define LCD_c 0b01100011				//c
#define LCD_d 0b01100100				//d
#define LCD_e 0b01100101				//e
#define LCD_f 0b01100110				//f
#define LCD_g 0b01100111				//g
#define LCD_h 0b01101000				//h
#define LCD_i 0b01101001				//i
#define LCD_j 0b01101010				//j
#define LCD_k 0b01101011				//k
#define LCD_l 0b01101100				//l
#define LCD_m 0b01101101				//m
#define LCD_n 0b01101110				//n
#define LCD_o 0b01101111				//o
#define LCD_p 0b01110000				//p
#define LCD_q 0b01110001				//q
#define LCD_r 0b01110010				//r
#define LCD_s 0b01110011				//s
#define LCD_t 0b01110100				//t
#define LCD_u 0b01110101				//u
#define LCD_v 0b01110110				//v
#define LCD_w 0b01110111				//w
#define LCD_x 0b01111000				//x
#define LCD_y 0b01111001				//y
#define LCD_z 0b01111010				//z
#define LCD_OPENBRACE 0b01111011		//open brace {
#define LCD_VBAR 0b01111100				//vertical bar |
#define LCD_CLOSEBRACE 0b01111101		//close brack }
#define LCD_RARROW 0b01111110			//right arrow ->
#define LCD_LARROW 0b01111111			//left arrow <-
#define LCD_DEGREE 0b11011111			//degree symbol 


/** Constants ********************************************************/
/* See IO_cfg.h */

/** PROTOTYPES *******************************************************/
void Debounce_Delay(unsigned int duration);
void LCD_Init(void);
void LCD_Bits(unsigned char data);
void LCD_Instruction(unsigned char command);
void LCD_wData(unsigned char data);
void LCD_Set_DDRAM(unsigned char);
unsigned int LCD_Busy(void);

/** EOF LCD.h *******************************************************/

#endif //__LCD_H